@@ -39,7 +39,7 @@ INTERNAL int __comedi_init=0;
 INTERNAL void initialize(void)
 {
 	char *s;
-	
+
 	__comedi_init=1;
 
 	if( (s=getenv("COMEDILIB_LOGLEVEL")) ){
@@ -47,7 +47,7 @@ INTERNAL void initialize(void)
 		COMEDILIB_DEBUG(3,"setting loglevel to %d\n",__comedi_loglevel);
 	}
 }
-  
+
 EXPORT_ALIAS_DEFAULT(_comedi_open,comedi_open,0.7.18);
 comedi_t* _comedi_open(const char *fn)
 {
@@ -65,12 +65,13 @@ comedi_t* _comedi_open(const char *fn)
 		goto cleanup;
 	}
 
-	if(comedi_ioctl(it->fd, COMEDI_DEVINFO, (unsigned long)&it->devinfo)<0)
+	if(comedi_ioctl(it->fd, COMEDI_DEVINFO, &it->devinfo) < 0)
 		goto cleanup;
 
 	it->n_subdevices=it->devinfo.n_subdevs;
 
-	get_subdevices(it);
+	if(get_subdevices(it) < 0)
+		goto cleanup;
 
 	it->magic=COMEDILIB_MAGIC;
 
@@ -86,8 +87,10 @@ EXPORT_ALIAS_DEFAULT(_comedi_close,comedi_close,0.7.18);
 int _comedi_close(comedi_t *it)
 {
 	subdevice *s;
-	int i;
+	int i,j;
 
+	if(!valid_dev(it))
+		return -1;
 	it->magic=0;
 
 	for(i=0;i<it->n_subdevices;i++){
@@ -103,6 +106,8 @@ int _comedi_close(comedi_t *it)
 		}
 		if(s->subd_flags&SDF_RANGETYPE){
 			free(s->range_type_list);
+			for(j=0;j<s->n_chan;j++)
+				free(s->rangeinfo_list[j]);
 			free(s->rangeinfo_list);
 		}else{
 			free(s->rangeinfo);
@@ -121,38 +126,39 @@ int _comedi_close(comedi_t *it)
 EXPORT_ALIAS_DEFAULT(_comedi_cancel,comedi_cancel,0.7.18);
 int _comedi_cancel(comedi_t *it,unsigned int subdevice)
 {
-	return comedi_ioctl(it->fd,COMEDI_CANCEL,subdevice);
+	if(!valid_dev(it)) return -1;
+	return comedi_ioctl(it->fd, COMEDI_CANCEL, (void*)(unsigned long)subdevice);
 }
 
 EXPORT_ALIAS_DEFAULT(_comedi_poll,comedi_poll,0.7.18);
 int _comedi_poll(comedi_t *it,unsigned int subdevice)
 {
-	return comedi_ioctl(it->fd,COMEDI_POLL,subdevice);
+	if(!valid_dev(it)) return -1;
+	return comedi_ioctl(it->fd, COMEDI_POLL, (void*)(unsigned long)subdevice);
 }
 
 EXPORT_ALIAS_DEFAULT(_comedi_fileno,comedi_fileno,0.7.18);
 int _comedi_fileno(comedi_t *it)
 {
-	if(!it)
-		return -1;
-
+	if(!valid_dev(it)) return -1;
 	return it->fd;
 }
 
 EXPORT_ALIAS_DEFAULT(_comedi_trigger,comedi_trigger,0.7.18);
 int _comedi_trigger(comedi_t *it,comedi_trig *t)
 {
-	if(!it || !t)
+	if(!valid_dev(it) || !t)
 		return -1;
 
-	return comedi_ioctl(it->fd, COMEDI_TRIG, (unsigned long)t);
+	return comedi_ioctl(it->fd, COMEDI_TRIG, t);
 }
 
 EXPORT_ALIAS_DEFAULT(_comedi_command,comedi_command,0.7.18);
 int _comedi_command(comedi_t *it,comedi_cmd *t)
 {
 	int ret;
-	ret = comedi_ioctl(it->fd, COMEDI_CMD, (unsigned long)t);
+	if(!valid_dev(it)) return -1;
+	ret = comedi_ioctl(it->fd, COMEDI_CMD, t);
 	__comedi_errno = errno;
 	switch(__comedi_errno){
 	case EIO:
@@ -166,7 +172,8 @@ EXPORT_ALIAS_DEFAULT(_comedi_command_test,comedi_command_test,0.7.18);
 int _comedi_command_test(comedi_t *it,comedi_cmd *t)
 {
 	int ret;
-	ret = comedi_ioctl(it->fd, COMEDI_CMDTEST, (unsigned long)t);
+	if(!valid_dev(it)) return -1;
+	ret = comedi_ioctl(it->fd, COMEDI_CMDTEST, t);
 	__comedi_errno = errno;
 	switch(__comedi_errno){
 	case EIO:
@@ -180,7 +187,8 @@ EXPORT_ALIAS_DEFAULT(_comedi_do_insnlist,comedi_do_insnlist,0.7.18);
 int _comedi_do_insnlist(comedi_t *it,comedi_insnlist *il)
 {
 	int ret;
-	ret = comedi_ioctl(it->fd, COMEDI_INSNLIST, (unsigned long)il);
+	if(!valid_dev(it)) return -1;
+	ret = comedi_ioctl(it->fd, COMEDI_INSNLIST, il);
 	__comedi_errno = errno;
 	return ret;
 }
@@ -188,8 +196,9 @@ int _comedi_do_insnlist(comedi_t *it,comedi_insnlist *il)
 EXPORT_ALIAS_DEFAULT(_comedi_do_insn,comedi_do_insn,0.7.18);
 int _comedi_do_insn(comedi_t *it,comedi_insn *insn)
 {
+	if(!valid_dev(it)) return -1;
 	if(it->has_insn_ioctl){
-		return comedi_ioctl(it->fd, COMEDI_INSN, (unsigned long)insn);
+		return comedi_ioctl(it->fd, COMEDI_INSN, insn);
 	}else{
 		comedi_insnlist il;
 		int ret;
@@ -197,7 +206,7 @@ int _comedi_do_insn(comedi_t *it,comedi_insn *insn)
 		il.n_insns = 1;
 		il.insns = insn;
 
-		ret = comedi_ioctl(it->fd, COMEDI_INSNLIST, (unsigned long)&il);
+		ret = comedi_ioctl(it->fd, COMEDI_INSNLIST, &il);
 
 		if(ret<0)return ret;
 		return insn->n;
@@ -207,12 +216,14 @@ int _comedi_do_insn(comedi_t *it,comedi_insn *insn)
 EXPORT_ALIAS_DEFAULT(_comedi_lock,comedi_lock,0.7.18);
 int _comedi_lock(comedi_t *it,unsigned int subdevice)
 {
-	return comedi_ioctl(it->fd, COMEDI_LOCK, subdevice);
+	if(!valid_dev(it)) return -1;
+	return comedi_ioctl(it->fd, COMEDI_LOCK, (void*)(unsigned long)subdevice);
 }
 
 EXPORT_ALIAS_DEFAULT(_comedi_unlock,comedi_unlock,0.7.18);
 int _comedi_unlock(comedi_t *it,unsigned int subdevice)
 {
-	return comedi_ioctl(it->fd, COMEDI_UNLOCK, subdevice);
+	if(!valid_dev(it)) return -1;
+	return comedi_ioctl(it->fd, COMEDI_UNLOCK, (void*)(unsigned long)subdevice);
 }
 
